@@ -1,9 +1,7 @@
 #include <limits>
 #include <cstdlib>
 #include <ctype.h>
-#define R_NO_REMAP
-#include <R.h>
-#include <Rinternals.h>
+#include "omxDefines.h"
 #include "omxState.h"
 #include "omxMatrix.h"
 #include "glue.h"
@@ -12,6 +10,10 @@
 
 #include "nlopt.h"
 
+#ifdef SHADOW_DIAG
+#pragma GCC diagnostic warning "-Wshadow"
+#endif
+
 namespace SLSQP {
 
 	struct context {
@@ -19,7 +21,7 @@ namespace SLSQP {
 		int origeq;
 		int eqredundent;
 		std::vector<bool> eqmask;
-		context(GradientOptimizerContext &goc) : goc(goc) {
+		context(GradientOptimizerContext &_goc) : goc(_goc) {
 			eqredundent = 0;
 		};
 	};
@@ -27,7 +29,7 @@ namespace SLSQP {
 struct fit_functional {
 	GradientOptimizerContext &goc;
 
-	fit_functional(GradientOptimizerContext &goc) : goc(goc) {};
+	fit_functional(GradientOptimizerContext &_goc) : goc(_goc) {};
 
 	double operator()(double *x, int thrId) const {
 		int mode = 0;
@@ -42,7 +44,6 @@ static double nloptObjectiveFunction(unsigned n, const double *x, double *grad,
 	int mode = grad != 0;
 	double fit = goc->solFun((double*) x, &mode);
 	if (grad) {
-		R_CheckUserInterrupt();
 		if (goc->maxMajorIterations != -1 && goc->getIteration() >= goc->maxMajorIterations) {
 			nlopt_force_stop(opt);
 		}
@@ -62,9 +63,8 @@ static double nloptObjectiveFunction(unsigned n, const double *x, double *grad,
 	Eigen::Map< Eigen::VectorXd > Egrad(grad, n);
 	if (goc->getWanted() & FF_COMPUTE_GRADIENT) {
 		Egrad = goc->grad;
-	} else if (goc->inConfidenceIntervalProblem()) {
-		Egrad.setZero();
-		Egrad[ goc->getConfidenceIntervalVarIndex() ] = goc->inConfidenceIntervalLowerBound()? 1 : -1;
+	} else if (goc->hasKnownGradient()) {
+		goc->setKnownGradient(Egrad);
 		goc->grad = Egrad;
 	} else {
 		if (goc->verbose >= 3) mxLog("fd_gradient start");
@@ -83,7 +83,7 @@ static double nloptObjectiveFunction(unsigned n, const double *x, double *grad,
 struct equality_functional {
 	GradientOptimizerContext &goc;
 
-	equality_functional(GradientOptimizerContext &goc) : goc(goc) {};
+	equality_functional(GradientOptimizerContext &_goc) : goc(_goc) {};
 
 	template <typename T1, typename T2>
 	void operator()(Eigen::MatrixBase<T1> &x, Eigen::MatrixBase<T2> &result) const {
@@ -103,6 +103,7 @@ static void nloptEqualityFunction(unsigned m, double* result, unsigned n, const
 	equality_functional ff(goc);
 	ff(Epoint, Eresult);
 	if (grad) {
+		goc.eqNorm = Eresult.array().abs().sum();
 		fd_jacobian(goc.gradientAlgo, goc.gradientIterations, goc.gradientStepSize,
 			    ff, Eresult, Epoint, jacobian);
 		if (ctx.eqmask.size() == 0) {
@@ -112,7 +113,7 @@ static void nloptEqualityFunction(unsigned m, double* result, unsigned n, const
 					bool match = (Eresult[c1] == Eresult[c2] &&
 						      (jacobian.col(c1) == jacobian.col(c2)));
 					if (match && !ctx.eqmask[c2]) {
-						ctx.eqmask[c2] = match;
+						ctx.eqmask[c2] = true;
 						++ctx.eqredundent;
 						if (goc.verbose >= 2) {
 							mxLog("nlopt: eq constraint %d is redundent with %d",
@@ -121,6 +122,16 @@ static void nloptEqualityFunction(unsigned m, double* result, unsigned n, const
 					}
 				}
 			}
+			for (int c1=0; c1 < int(m); ++c1) {
+				if (ctx.eqmask[c1]) continue;
+				if ((jacobian.col(c1).array() == 0.0).all()) {
+					ctx.eqmask[c1] = true;
+					++ctx.eqredundent;
+					if (goc.verbose >= 2) {
+						mxLog("nlopt: eq constraint %d is never active", c1);
+					}
+				}
+			}
 			if (ctx.eqredundent) {
 				if (goc.verbose >= 1) {
 					mxLog("nlopt: detected %d redundent equality constraints; retrying",
@@ -142,8 +153,7 @@ static void nloptEqualityFunction(unsigned m, double* result, unsigned n, const
 		}
 		++dx;
 	}
-	if (goc.verbose >= 4 && grad) {
-		mxPrintMat("eq result", Uresult);
+	if (goc.verbose >= 3 && grad) {
 		mxPrintMat("eq jacobian", Ujacobian);
 	}
 }
@@ -151,7 +161,7 @@ static void nloptEqualityFunction(unsigned m, double* result, unsigned n, const
 struct inequality_functional {
 	GradientOptimizerContext &goc;
 
-	inequality_functional(GradientOptimizerContext &goc) : goc(goc) {};
+	inequality_functional(GradientOptimizerContext &_goc) : goc(_goc) {};
 
 	template <typename T1, typename T2>
 	void operator()(Eigen::MatrixBase<T1> &x, Eigen::MatrixBase<T2> &result) const {
@@ -167,16 +177,13 @@ static void nloptInequalityFunction(unsigned m, double *result, unsigned n, cons
 	Eigen::Map< Eigen::VectorXd > Epoint((double*)x, n);
 	Eigen::Map< Eigen::VectorXd > Eresult(result, m);
 	Eigen::Map< Eigen::MatrixXd > jacobian(grad, n, m);
-	if (grad && goc->verbose >= 2) {
-		if (m == 1) {
-			mxLog("major iteration ineq=%.12f", Eresult[0]);
-		} else {
-			mxPrintMat("major iteration ineq", Eresult);
-		}
-	}
 	inequality_functional ff(*goc);
 	ff(Epoint, Eresult);
+	if (grad && goc->verbose >= 2) {
+		mxPrintMat("major iteration ineq", Eresult);
+	}
 	if (grad) {
+		goc->ineqNorm = Eresult.array().abs().sum();
 		fd_jacobian(goc->gradientAlgo, goc->gradientIterations, goc->gradientStepSize,
 			    ff, Eresult, Epoint, jacobian);
 		if (!std::isfinite(Eresult.sum())) {
@@ -211,17 +218,11 @@ void omxInvokeNLOPT(GradientOptimizerContext &goc)
         nlopt_set_upper_bounds(opt, goc.solUB.data());
 
 	int eq, ieq;
-	globalState->countNonlinearConstraints(eq, ieq);
+	globalState->countNonlinearConstraints(eq, ieq, false);
 
-	if (goc.inConfidenceIntervalProblem()) {
-		nlopt_set_xtol_rel(opt, 5e-3);
-		std::vector<double> tol(goc.numFree, std::numeric_limits<double>::epsilon());
-		nlopt_set_xtol_abs(opt, tol.data());
-	} else {
-		// The *2 is there to roughly equate accuracy with NPSOL.
-		nlopt_set_ftol_rel(opt, goc.ControlTolerance * 2);
-		nlopt_set_ftol_abs(opt, std::numeric_limits<double>::epsilon());
-	}
+	// The *2 is there to roughly equate accuracy with NPSOL.
+	nlopt_set_ftol_rel(opt, goc.ControlTolerance * 2);
+	nlopt_set_ftol_abs(opt, std::numeric_limits<double>::epsilon());
         
 	nlopt_set_min_objective(opt, SLSQP::nloptObjectiveFunction, &goc);
 
@@ -230,12 +231,14 @@ void omxInvokeNLOPT(GradientOptimizerContext &goc)
         if (eq + ieq) {
 		ctx.origeq = eq;
                 if (ieq > 0){
+			if (goc.verbose >= 2) mxLog("%d inequality constraints", ieq);
 			goc.inequality.resize(ieq);
 			std::vector<double> tol(ieq, feasibilityTolerance);
 			nlopt_add_inequality_mconstraint(opt, ieq, SLSQP::nloptInequalityFunction, &goc, tol.data());
                 }
                 
                 if (eq > 0){
+			if (goc.verbose >= 2) mxLog("%d equality constraints", eq);
 			goc.equality.resize(eq);
 			std::vector<double> tol(eq, feasibilityTolerance);
 			nlopt_add_equality_mconstraint(opt, eq, SLSQP::nloptEqualityFunction, &ctx, tol.data());
@@ -272,15 +275,18 @@ void omxInvokeNLOPT(GradientOptimizerContext &goc)
 			goc.informOut = INFORM_ITERATION_LIMIT;
 		}
 	} else if (code == NLOPT_ROUNDOFF_LIMITED) {
-		if (goc.getIteration() - priorIterations <= 2) {
+		if (goc.eqNorm > feasibilityTolerance || goc.ineqNorm > feasibilityTolerance) {
+			goc.informOut = INFORM_NONLINEAR_CONSTRAINTS_INFEASIBLE;
+		} else if (goc.getIteration() - priorIterations <= 2) {
 			Rf_error("%s: Failed due to singular matrix E or C in LSQ subproblem or "
 				 "rank-deficient equality constraint subproblem or "
-				 "positive directional derivative in line search", goc.optName);
+				 "positive directional derivative in line search "
+				 "(eq %.4g ineq %.4g)", goc.optName, goc.eqNorm, goc.ineqNorm);
 		} else {
 			goc.informOut = INFORM_NOT_AT_OPTIMUM;  // is this correct? TODO
 		}
 	} else if (code < 0) {
-		Rf_error("NLOPT fatal error %d", code);
+		Rf_error("NLOPT unrecognized error %d; please report to developers", code);
 	} else if (code == NLOPT_MAXEVAL_REACHED) {
 		goc.informOut = INFORM_ITERATION_LIMIT;
 	} else {
